/* This file is part of The_Merry_Turnip
 * 
 * The_Merry_Turnip - TODO: Description
 * Copyright © DAVY Guillaume 2009 <davyg2@gmail.com>
 * 
 * The_Merry_Turnip is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The_Merry_Turnip is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*!	\file coeur.h
	\brief Definition of class chap
*/

#ifndef __COEUR_H_INCLUDED__
#define __COEUR_H_INCLUDED__

//!Main class containing all important call to the irrlicht and a lot of utilities, derive from base
class coeur : public base
{
public:
	//! Constructor
	coeur();
	
	//! Destructor
	~coeur();

	//! Init the coeur
	int init(startmenu*);
	
	//! Updating (must be call on each loop)
	int update();
	
	//! Close the coeur
	int close();

	//! Set the title of the window
	int setCaption(stringw);
	
	//! Add a mesh to irrlicht
	IAnimatedMeshSceneNode* addMesh(stringc);
	
	//! Add a light to irrlicht
	ILightSceneNode* addLight(int);
	
	//! Load a texture
	ITexture* getTexture(stringc);
	
	//! Return the dimension of the window
	dimension2d<s32> getDim();
	
	//! Return wich actionMesh the user is looking at
	actionMesh *getSelectedAction();
	
	//! Return an Item from his Id
	item getItem(int);
	
	//! Return an ActionMesh from his id
	actionMesh* getActionMesh(int);
	
	//! Return an ActionMesh from his node
	actionMesh* getActionMesh(ISceneNode*);
	
	//! Return the localItem of an observationMesh from his id
	localItem* getObservationMesh(int);
	
	//! Return the localItem of an observationItem from his id
	localItem* getObservationItem(int);
	
	//! Return the localItem corresponding to a failing combinaison
	localItem* getCombinaison();
	
	//! Return the postion of the cursor
	position2d<s32> getCurPos();
	
	//! Return the current time
	u32 getTime();
	
	//! Return the version of the soft
	stringc getVersion();
	
	//! Play a sound
	int play(stringc);
	
	//! Launch the animation of an actionMesh
	int anim(int,int,int);
	
	//! Set the skybox
	int setSkyBox(stringc,stringc,stringc,stringc,stringc,stringc);
	
	//! Called when an event occured
	virtual bool OnEvent(const SEvent&);
	
	//! Return the pointer to the logger
	logger* getLogger();
	
	//! Launch the Credit
	int startCredit();
	
	//! Load a new chapter
	int loadChap(int);
	
	//! Load a new lieu
	int loadLieu(int);
	
	//! Load a new local
	int loadLocal(int);
	
	//! The console has all right on this class
	friend class console;
	
	//! The python interface has all right on this class
	friend class pyInt;
	
	IParticleSystemSceneNode* addParticleSystem();
	
private:
	
	//! Get Selected Node
	ISceneNode* getSelectedNode();
	
	//! Pick a node
	void pickNode(ISceneNode*,line3df,f32&,ISceneNode*&);
	
	//! The Irrlicht Device
	IrrlichtDevice *device;
	
	//! The Irrlicht Driver
	IVideoDriver* driver;
	public:
	//! The Irrlicht SceneManager
	ISceneManager* smgr;
	private:
	//! The Irrlicht Envirronement
	IGUIEnvironment* env;
	public:
	//! The Camera
	ICameraSceneNode* cam;
	private:
	//! The driver chosen
	E_DRIVER_TYPE dr;
	
	//! The dimension chosen
	dimension2d<s32> dim;
	
	//! The bits o the window
	int bits;
	
	//! Does the render take all the screen
	bool fullscreen;
	
	//! Does it draw the stencil shadow
	bool stencilbuffer;
	
	//! Does the certical synchronisation is activated
	bool vsync;
	
	//! Language of the game loaded
	int lang;
	
	//! Pointer to the interface class
	interface* in;
	
	//! Pointer to the console class
	console* co;
	
	//! Pointer to the credit class
	credit* mycredit;

	logger* mylogger;
	
	//! Set to true if the coeur must be quit
	bool q;
	
	//! Position of the cursor
	position2d<s32> curPos;
	
	//! List containing the path of all the chapters
	std::vector<stringc*>* pathChap;
	
	//! List containing the path of all the lieu
	std::vector<stringc*>* pathLieu;
	
	//! List containing the path of all the local
	std::vector<stringc*>* pathLocal;
	
	//! Pointer to the chap class
	chap* currentChap;
	
	//! Pointer to the lieu class
	lieu* currentLieu;
	
	//! Pointer to the local class
	local* currentLocal;
	
	pyInt* py;
	
	//! Pointer to the actionMesh selected
	actionMesh *selectedAction;
	
	//! Pointer to the metaSelector
	IMetaTriangleSelector* metaSelector;
	
	//! Folder containg all the data of the game
	stringc pathGame;
	
	//! True if the coeur must draw the scene
	bool activeScene;
	
	//! True if the coeur must draw the interface
	bool activeIn;
	
#ifdef AUDIO
	//! The IrrKlang Sound Engine
	ISoundEngine* engine;
#endif
};// End class coeur

#endif //__COEUR_H_INCLUDED__

